@@ -22,7 +22,7 @@ inactive
 {'full_text': 'No activity'}
 """
 
-STRING_NOT_INSTALLED = "isn't installed"
+STRING_NOT_INSTALLED = "not installed"
 
 
 class Py3status:
