@@ -4,6 +4,7 @@ Display tasks currently running in taskwarrior.
 
 Configuration parameters:
     cache_timeout: refresh interval for this module (default 5)
+    filter: specify one or more criteria to use (default 'status:pending')
     format: display format for this module (default '{task}')
 
 Format placeholders:
@@ -20,7 +21,7 @@ SAMPLE OUTPUT
 """
 
 import json
-STRING_NOT_INSTALLED = "not installed"
+STRING_NOT_INSTALLED = 'not installed'
 
 
 class Py3status:
@@ -28,18 +29,22 @@ class Py3status:
     """
     # available configuration parameters
     cache_timeout = 5
+    filter = 'status:pending'
     format = '{task}'
 
     def post_config_hook(self):
         if not self.py3.check_commands('task'):
             raise Exception(STRING_NOT_INSTALLED)
+        if self.filter:
+            self.task_command = 'task %s export' % self.filter
+        else:
+            self.task_command = 'task export'
 
     def taskWarrior(self):
         def describeTask(taskObj):
             return str(taskObj['id']) + ' ' + taskObj['description']
 
-        task_command = 'task start.before:tomorrow status:pending export'
-        task_json = json.loads(self.py3.command_output(task_command))
+        task_json = json.loads(self.py3.command_output(self.task_command))
         task_result = ', '.join(map(describeTask, task_json))
         return {
             'cached_until': self.py3.time_in(self.cache_timeout),
