@@ -5,7 +5,7 @@ Display public IP address and online status.
 Configuration parameters:
     button_refresh: mouse button to refresh this module (default 2)
     button_toggle: mouse button to toggle between states (default 1)
-    cache_timeout: how often we refresh this module in seconds (default 30)
+    cache_timeout: how often we refresh this module in seconds (default 60)
     expected: define expected values for format placeholders,
         and use `color_degraded` to show the output of this module
         if any of them does not match the actual value.
@@ -19,15 +19,17 @@ Configuration parameters:
     icon_on: what to display when online (default '●')
     mode: default mode to display is 'ip' or 'status' (click to toggle)
         (default 'ip')
-    negative_cache_timeout: how often to check again when offline (default 2)
     timeout: how long before deciding we're offline (default 5)
     url_geo: IP to check for geo location (must output json)
-        (default 'https://freegeoip.net/json/')
+        (default 'https://ifconfig.co/json')
 
 Format placeholders:
-    {icon} display the icon
-    {country} display the country
-    {ip} display current ip address
+    {icon}        eg ●, ■
+    {country}     eg France
+    {country_iso} eg FR
+    {ip}          eg 123.45.67.890
+    {ip_decimal}  eg 1234567890
+    {city}        eg Paris
     any other key in JSON fetched from `url_geo`
 
 Color options:
@@ -35,20 +37,20 @@ Color options:
     color_degraded: Output is unexpected (IP/country mismatch, etc.)
     color_good: Online
 
-@author ultrabug
+@author ultrabug, Cyril Levis (@cyrinux)
 
 SAMPLE OUTPUT
 {'full_text': '37.48.108.0'}
 
 geo
-{'full_text': '37.48.108.0 Russia'}
+{'full_text': '37.48.108.0 France'}
 
 mode
 {'color': '#00FF00', 'full_text': u'\u25cf'}
 """
 
 URL_GEO_OLD_DEFAULT = 'http://ip-api.com/json'
-URL_GEO_NEW_DEFAULT = 'https://freegeoip.net/json/'
+URL_GEO_NEW_DEFAULT = 'https://ifconfig.co/json'
 
 
 class Py3status:
@@ -57,14 +59,13 @@ class Py3status:
     # available configuration parameters
     button_refresh = 2
     button_toggle = 1
-    cache_timeout = 30
+    cache_timeout = 60
     expected = None
     format = '{ip}'
     hide_when_offline = False
     icon_off = u'■'
     icon_on = u'●'
     mode = 'ip'
-    negative_cache_timeout = 2
     timeout = 5
     url_geo = URL_GEO_NEW_DEFAULT
 
@@ -75,6 +76,10 @@ class Py3status:
                     'param': 'url',
                     'msg': 'obsolete parameter, use `url_geo` instead',
                 },
+                {
+                    'param': 'negative_cache_timeout',
+                    'msg': 'obsolete parameter',
+                },
             ],
             'rename': [
                 {
@@ -97,7 +102,7 @@ class Py3status:
         # Backwards compatibility
         self.substitutions = {}
         if self.url_geo == URL_GEO_NEW_DEFAULT:
-            self.substitutions['country'] = 'country_name'
+            self.substitutions['country_code'] = 'country_iso'
         elif self.url_geo == URL_GEO_OLD_DEFAULT:
             self.substitutions['ip'] = 'query'
 
@@ -131,14 +136,13 @@ class Py3status:
         """
         info = self._get_my_ip_info()
         response = {
-            'cached_until': self.py3.time_in(self.negative_cache_timeout)
+            'cached_until': self.py3.time_in(self.cache_timeout)
         }
 
         if info is None and self.hide_when_offline:
             response['full_text'] = ''
         elif info is not None:
             info['icon'] = self.icon_on
-            response['cached_until'] = self.py3.time_in(self.cache_timeout)
             response['color'] = self.py3.COLOR_GOOD
             for key, val in self.expected.items():
                 if val != info.get(key):
