@@ -17,9 +17,10 @@ Configuration parameters:
         (default True)
     signal_bad: Bad signal strength in percent (default 29)
     signal_degraded: Degraded signal strength in percent (default 49)
-    use_sudo: Use sudo to run iw, make sure iw requires some root rights
-        without a password by adding a sudoers entry, eg...
-        '<user> ALL=(ALL) NOPASSWD:/usr/bin/iw dev,/usr/bin/iw dev [a-z]* link'
+    use_sudo: Use sudo to run iw and ip. make sure to give some root rights
+        to run without a password by editing the sudoers file, eg...
+        '<user> ALL=(ALL) NOPASSWD:/sbin/iw dev,/sbin/iw dev [a-z]* link'
+        '<user> ALL=(ALL) NOPASSWD:/sbin/ip addr list [a-z]*'
         (default False)
 
 Format placeholders:
@@ -77,14 +78,11 @@ class Py3status:
     def post_config_hook(self):
         self._max_bitrate = 0
         self._ssid = ''
-        self.iw_cmd = self.py3.check_commands(['iw', '/sbin/iw'])
-        # Try and guess the wifi interface
-        cmd = [self.iw_cmd, 'dev']
-        if self.use_sudo:
-            cmd.insert(0, 'sudo')
+        iw = self.py3.check_commands(['iw', '/sbin/iw'])
+        # get wireless interface
         try:
-            iw = self.py3.command_output(cmd)
-            devices = re.findall('Interface\s*([^\s]+)', iw)
+            data = self.py3.command_output([iw, 'dev'])
+            devices = re.findall('Interface\s*([^\s]+)', data)
             if not devices or 'wlan0' in devices:
                 self.device = 'wlan0'
             else:
@@ -92,6 +90,13 @@ class Py3status:
         except:
             pass
 
+        self.iw_dev_id_link = [iw, 'dev', self.device, 'link']
+        self.ip_addr_list_id = ['ip', 'addr', 'list', self.device]
+        # use_sudo?
+        if self.use_sudo:
+            for cmd in [self.iw_dev_id_link, self.ip_addr_list_id]:
+                cmd[0:0] = ['sudo', '-n']
+
         # DEPRECATION WARNING
         format_down = getattr(self, 'format_down', None)
         format_up = getattr(self, 'format_up', None)
@@ -114,11 +119,8 @@ class Py3status:
         """
         self.signal_dbm_bad = self._percent_to_dbm(self.signal_bad)
         self.signal_dbm_degraded = self._percent_to_dbm(self.signal_degraded)
-        cmd = [self.iw_cmd, 'dev', self.device, 'link']
-        if self.use_sudo:
-            cmd.insert(0, 'sudo')
         try:
-            iw = self.py3.command_output(cmd)
+            iw = self.py3.command_output(self.iw_dev_id_link)
         except:
             return {'cache_until': self.py3.CACHE_FOREVER,
                     'color': self.py3.COLOR_ERROR or self.py3.COLOR_BAD,
@@ -148,7 +150,7 @@ class Py3status:
         if ssid_out:
             ssid = ssid_out.group(1)
             # `iw` command would prints unicode SSID like `\xe8\x8b\x9f`
-            # the the `ssid` here would be '\\xe8\\x8b\\x9f' (note the escape)
+            # the `ssid` here would be '\\xe8\\x8b\\x9f' (note the escape)
             # it needs to be decoded using 'unicode_escape', to '苟'
             ssid = ssid.encode('latin-1').decode('unicode_escape')
             ssid = ssid.encode('latin-1').decode('utf-8')
@@ -157,10 +159,7 @@ class Py3status:
 
         # check command
         if self.py3.format_contains(self.format, 'ip'):
-            cmd = ['ip', 'addr', 'list', self.device]
-            if self.use_sudo:
-                cmd.insert(0, 'sudo')
-            ip_info = self.py3.command_output(cmd)
+            ip_info = self.py3.command_output(self.ip_addr_list_id)
             ip_match = re.search('inet\s+([0-9.]+)', ip_info)
             if ip_match:
                 ip = ip_match.group(1)
