@@ -18,6 +18,8 @@ For convenience, this module also proposes some added features:
 
 Configuration parameters:
     cache_timeout: how often to (re)detect the outputs (default 10)
+    command: a custom command to be run after display configuration changes
+        (default None)
     fallback: when the current output layout is not available anymore,
         fallback to this layout if available. This is very handy if you
         have a laptop and switched to an external screen for presentation
@@ -65,15 +67,27 @@ Dynamic configuration parameters:
         Example: DP1_workspaces = "1,2,3"
     <OUTPUT>_rotate: rotate the output as told
         Example: DP1_rotate = "left"
+    <OUTPUT>_mode: define the mode (resolution) for the output
+                   if not specified use --auto : prefered mode
+        Example: eDP1_mode = "2560x1440
 
 Color options:
     color_bad: Displayed layout unavailable
     color_degraded: Using a fallback layout
     color_good: Displayed layout active
 
-Example config:
+Notes:
+    Some days are just bad days. Running `xrandr --query` command can
+    cause unexplainable brief screen freezes due to an overall combination
+    of computer hardware, installed software, your choice of linux distribution,
+    and/or some other unknown factors such as recent system updates.
 
+    Configuring `cache_timeout` with a different number, eg `3600` (an hour)
+    or `-1` (runs once) can be used to remedy this issue. See issue #580.
+
+Examples:
 ```
+# start with a preferable setup
 xrandr {
     force_on_start = "eDP1+DP1"
     DP1_pos = "left-of eDP1"
@@ -108,6 +122,7 @@ class Py3status:
     """
     # available configuration parameters
     cache_timeout = 10
+    command = None
     fallback = True
     fixed_width = True
     force_on_start = None
@@ -160,17 +175,20 @@ class Py3status:
         for line in current.splitlines():
             try:
                 s = line.split(' ')
+                infos = line[line.find('('):]
                 if s[1] == 'connected':
-                    output, state = s[0], s[1]
-                    if s[2][0] == '(':
-                        mode, infos = None, ' '.join(s[2:]).strip('\n')
-                    else:
-                        mode, infos = s[2], ' '.join(s[3:]).strip('\n')
-                        active_layout.append(output)
+                    output, state, mode = s[0], s[1], None
+                    for index, x in enumerate(s[2:], 2):
+                        if 'x' in x and '+' in x:
+                            mode = x
+                            active_layout.append(output)
+                            infos = line[line.find(s[index + 1]):]
+                            break
+                        elif '(' in x:
+                            break
                     connected.append(output)
                 elif s[1] == 'disconnected':
-                    output, state = s[0], s[1]
-                    mode, infos = None, ' '.join(s[2:]).strip('\n')
+                    output, state, mode = s[0], s[1], None
                     disconnected.append(output)
                 else:
                     continue
@@ -288,20 +306,25 @@ class Py3status:
             if output in combination:
                 pos = getattr(self, '{}_pos'.format(output), '0x0')
                 rotation = getattr(self, '{}_rotate'.format(output), 'normal')
+                resolution = getattr(self, '{}_mode'.format(output), None)
+                resolution = '--mode {}'.format(resolution) if resolution else '--auto'
                 if rotation not in ['inverted', 'left', 'normal', 'right']:
                     self.py3.log('configured rotation {} is not valid'.format(
                         rotation))
                     rotation = 'normal'
                 #
                 if mode == 'clone' and previous_output is not None:
-                    cmd += ' --auto --same-as {}'.format(previous_output)
+                    cmd += ' {} --same-as {}'.format(resolution,
+                                                     previous_output)
                 else:
                     if ('above' in pos or 'below' in pos or 'left-of' in pos or
                             'right-of' in pos):
-                        cmd += ' --auto --{} --rotate {}'.format(pos, rotation)
+                        cmd += ' {} --{} --rotate {}'.format(resolution, pos,
+                                                             rotation)
                     else:
-                        cmd += ' --auto --pos {} --rotate {}'.format(pos,
-                                                                     rotation)
+                        cmd += ' {} --pos {} --rotate {}'.format(resolution,
+                                                                 pos,
+                                                                 rotation)
                 previous_output = output
             else:
                 cmd += ' --off'
@@ -313,6 +336,9 @@ class Py3status:
             self.active_mode = mode
         self.py3.log('command "{}" exit code {}'.format(cmd, code))
 
+        if self.command:
+            self.py3.command_run(self.command)
+
         # move workspaces to outputs as configured
         self._apply_workspaces(combination, mode)
 
