@@ -91,6 +91,32 @@ class ConfigParser:
             }
         }
 
+    * environment variable support
+
+        order += env(ORDER_VAR)
+
+        my_module {
+            my_guessed_type = env(MY_VAR)
+            my_str = env(MY_VAR, str)
+            my_int = env(MY_INT_VAR, int)
+            my_bool = env(MY_FLAG, bool)
+            my_complex = {
+                'list' : [1, 2, env(MY_LIST_ENTRY, int)],
+                'dict' : {'x': env(MY_DICT_VAL), 'y': 2}
+            }
+        }
+
+    * execute shell code
+
+        my_module {
+            my_guessed_type = shell(pass show git|head -n1)
+            my_str = shell(pass show git|head -n1, str)
+            my_int = shell(pass show git|head -n1, int)
+            my_bool = shell(pass show git|head -n1, bool)
+        }
+
+        (shell code may not include any parenthesis!)
+
     * quality feedback on parse errors.
         details include error description, line number, position.
 
@@ -98,6 +124,9 @@ class ConfigParser:
 
     TOKENS = [
         '#.*$'  # comments
+        # environment variables
+        '|(?P<env_var>env\(\s*([0-9a-zA-Z_]+)(\s*,\s*[a-zA-Z_]+)?\s*\))'
+        '|(?P<shell>shell\(.+?(\s*,\s*(int|float|str|bool|auto))?\))'  # shell code
         '|(?P<operator>[()[\]{},:]|\+?=)'  # operators
         '|(?P<literal>'
         r'("(?:[^"\\]|\\.)*")'  # double quoted string
@@ -213,6 +242,10 @@ class ConfigParser:
                 t_type = 'literal'
             elif token.group('newline'):
                 t_type = 'newline'
+            elif token.group('env_var'):
+                t_type = 'env_var'
+            elif token.group('shell'):
+                t_type = 'shell'
             elif token.group('unknown'):
                 t_type = 'unknown'
             else:
@@ -292,6 +325,68 @@ class ConfigParser:
             return None
         return value
 
+    def make_value_from_env(self, value):
+        # Extract the environment variable and type
+        match = re.match(
+            'env\(\s*([0-9a-zA-Z_]+)(\s*,\s*[a-zA-Z_]+)?\s*\)', value)
+        env_var, env_type = match.groups()
+        if not env_type:
+            env_type = 'auto'
+        else:
+            # Remove comma and whitespace from match
+            # i.e '   ,  my_conversion' -> 'my_conversion'
+            env_type = env_type.strip()[1:].lstrip()
+
+        conversion_options = {
+            'str': str,
+            'int': int,
+            'float': float,
+
+            # Treat booleans specially
+            'bool': (lambda val: val.lower() in ('true', '1')),
+
+            # Auto-guess the type
+            'auto': self.make_value,
+        }
+        if env_type not in conversion_options:
+            self.error('Invalid env conversion function in env_var')
+
+        # Check the environment variable
+        # TODO: dynamic support, (lambda: os.getenv(env_var))
+        value = os.getenv(env_var)
+        if value is None:
+            self.error('Environment variable \'%s\' undefined' % env_var)
+
+        try:
+            return conversion_options[env_type](value)
+        except ValueError:
+            self.error('Bad conversion')
+
+    def make_value_from_shell(self, value):
+        conversion_options = {
+            'int': int,
+            'float': float,
+            'str': str,
+
+            'bool': (lambda val: val.lower() in ('', 'true', '1')),
+            'auto': self.make_value,
+        }
+
+        match = re.match('shell\((.+?)(\s*,\s*(int|float|str|bool|auto))?\)', value)
+        shell, _, var_type = match.groups()
+        try:
+            shell_stdout = str(check_output(shell, shell=True).rstrip(), encoding='utf-8')
+        except CalledProcessError:
+            if var_type is not None and var_type.lower() == 'bool':
+                return False
+            self.error('shell script exited with an error')
+        if var_type is None:
+            return shell_stdout
+        try:
+            return conversion_options[var_type](shell_stdout)
+        except ValueError:
+            self.error('Bad conversion')
+
     def separator(self, separator=',', end_token=None):
         '''
         Read through tokens till the required separator is found.  We ignore
@@ -370,6 +465,10 @@ class ConfigParser:
                     continue
             if token['type'] == 'literal':
                 return self.make_value(t_value)
+            if token['type'] == 'env_var':
+                return self.make_value_from_env(t_value)
+            if token['type'] == 'shell':
+                return self.make_value_from_shell(t_value)
             elif t_value == '[':
                 return self.make_list()
             elif t_value == '{':
@@ -458,6 +557,10 @@ class ConfigParser:
                 if not name and not re.match('[a-zA-Z_]', value):
                     self.error('Invalid name')
                 name.append(value)
+            elif t_type == 'env_var':
+                self.error('Name expected')
+            elif t_type == 'shell':
+                self.error('Shell command expected')
             elif t_type == 'operator':
                 name = ' '.join(name)
                 if not name:
@@ -638,19 +741,23 @@ def process_config(config_path, py3_wrapper=None):
                 fixed[k] = v
         return fixed
 
+    def append_modules(item):
+        module_type = get_module_type(item)
+        if module_type == 'i3status':
+            if item not in i3s_modules:
+                i3s_modules.append(item)
+        else:
+            if item not in py3_modules:
+                py3_modules.append(item)
+
     def add_container_items(module_name):
         module = modules.get(module_name, {})
         items = module.get('items', [])
         for item in items:
             if item in config:
                 continue
-            module_type = get_module_type(item)
-            if module_type == 'i3status':
-                if item not in i3s_modules:
-                    i3s_modules.append(item)
-            else:
-                if item not in py3_modules:
-                    py3_modules.append(item)
+
+            append_modules(item)
             module = modules.get(item, {})
             config[item] = remove_any_contained_modules(module)
             # add any children
@@ -659,15 +766,10 @@ def process_config(config_path, py3_wrapper=None):
     # create config for modules in order
     for name in config_info.get('order', []):
         module = modules.get(name, {})
-        module_type = get_module_type(name)
         config['order'].append(name)
         add_container_items(name)
-        if module_type == 'i3status':
-            if name not in i3s_modules:
-                i3s_modules.append(name)
-        else:
-            if name not in py3_modules:
-                py3_modules.append(name)
+        append_modules(name)
+
         config[name] = remove_any_contained_modules(module)
 
     config['on_click'] = on_click
